import unittest

from xml.dom.minidom import Element, parseString

from harvestingkit.jats_utils import JATSParser


class JATSUtilsTests(unittest.TestCase):

    def setUp(self):
        self.jats_parser = JATSParser()
    
    def test_get_orcid(self):
        """
        See http://jats.nlm.nih.gov/archiving/tag-library/1.1d1/n-dsw0.html for orcid in contrib tag 
        """

        xml = parseString(
              """<contrib>
                   <contrib-id contrib-id-type="orcid">http://orcid.org/1792-3336-9172-961X</contrib-id>
                   <name><surname>Fauller</surname>
                   <given-names>Betty Lou</given-names>
                   </name>
                   <degrees>BA, MA</degrees>
                 </contrib>""")

        self.assertEqual(self.jats_parser._get_orcid(xml), '1792-3336-9172-961X')


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(JATSUtilsTests)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2014, 2015 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

import unittest

from harvestingkit.html_utils import MathMLParser


class HTMLUtilsTests(unittest.TestCase):

    """Tests for all utility functions."""

    def test_mathml(self):
        """Test that MathML is kept."""
        data = (u'In this paper we continue the study of Q -operators in'
                u' the six-vertex model and its higher spin generalizations.'
                u' In [1] we derived a new expression for the higher spin R'
                u' -matrix associated with the affine quantum algebra '
                u'<math xmlns="http://www.w3.org/1998/Math/MathML" altimg="si1.gif">'
                u'<msub><mrow><mi>U</mi></mrow><mrow><mi>q</mi></mrow></msub>'
                u'<mo stretchy="false">(</mo><mover accent="true"><mrow><mrow>'
                u'<mi mathvariant="italic">sl</mi></mrow><mo stretchy="false">'
                u'(</mo><mn>2</mn><mo stretchy="false">)</mo></mrow><mrow><mo>'
                u'^</mo></mrow></mover><mo stretchy="false">)</mo></math>'
                u' . Taking a special limit in this R -matrix we obtained new'
                u' formulas for the Q -operators acting in the tensor product'
                u' of representation spaces with arbitrary complex spin.')
        self.assertEqual(MathMLParser.html_to_text(data), data)

    def test_html(self):
        """Test that HTML is stripped."""
        data = (u'<p><roman>CH</roman><sub>3</sub><roman>NH</roman><sub>3</sub>'
                u'<roman>PbX</roman>(<roman>X</roman> = <roman>Br</roman>, '
                u'<roman>I</roman>, <roman>Cl</roman>) perovskites have recently'
                u'been used as light absorbers in hybrid organic-inorganic solid-state'
                u' solar cells, with efficiencies above 15%.</p>')
        expected_data = (u'CH3NH3PbX(X = Br, I, Cl) perovskites have recently'
                         u'been used as light absorbers in hybrid organic-inorganic solid-state '
                         u'solar cells, with efficiencies above 15%.')
        self.assertEqual(MathMLParser.html_to_text(data), expected_data)

    def test_htmlentity(self):
        """Test that HTML entities are kept."""
        data = "This &amp; that and &lt; is there."
        self.assertEqual(MathMLParser.html_to_text(data), data)

    def test_xml_encoding(self):
        """Test that HTML entities are kept."""
        data = "This & that and 2<y<3 is > there."
        expected_data = "This &amp; that and 2&lt;y&lt;3 is > there."
        self.assertEqual(MathMLParser.html_to_text(data), expected_data)

    def test_htmlentity_case(self):
        """Test that HTML entities are dealt with smartly."""
        data = (u'Project at CERN, Proc. of the Workshop on Future Directions in Detector R&D;')
        expected_data = (u'Project at CERN, Proc. of the Workshop on Future Directions in Detector R&amp;D;')
        self.assertEqual(MathMLParser.html_to_text(data), expected_data)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
##
## This file is part of Harvesting Kit.
## Copyright (C) 2013, 2014 CERN.
##
## Harvesting Kit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2 of the
## License, or (at your option) any later version.
##
## Harvesting Kit is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
## 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
"""
Set of utilities for the SCOAP3 project.
"""

from __future__ import division

import sys
import logging
import time

from tarfile import TarFile
from zipfile import ZipFile
try:
    from invenio.errorlib import register_exception
except ImportError:
    register_exception = lambda a=1, b=2: True

from os.path import join

try:
    from invenio.config import CFG_LOGDIR
except ImportError:
    from distutils.sysconfig import get_python_lib
    CFG_LOGDIR = join(get_python_lib(),
                      "harvestingkit",
                      "log")
CFG_CROSSREF_DOIS_PER_REQUEST = 10
CFG_CROSSREF_API_URL = "http://doi.crossref.org/search/doi?"


def lock_issue():
    """
    Locks the issu in case of error.
    """
    # TODO
    print >> sys.stderr, "locking issue"


# Creates a logger object
def create_logger(publisher,
                  filename=join(CFG_LOGDIR, 'scoap3_harvesting.log'),
                  logging_level=logging.DEBUG):
    logger = logging.getLogger(publisher)
    formatter = logging.Formatter(('%(asctime)s - %(name)s - '
                                   '%(levelname)-8s - %(message)s'))

    fh = logging.FileHandler(filename=filename)
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    ch = logging.StreamHandler()
    ch.setFormatter(formatter)
    logger.addHandler(ch)

    logger.setLevel(logging_level)

    return logger


def progress_bar(n):
    num = 0
    while num <= n:
        yield "%d [%s%s]" % (num/n*100, "="*num, '.'*(n-num))
        num += 1


class MD5Error(Exception):
    def __init__(self, value):
        self.value = value


class NoNewFiles(Exception):
    def __init__(self, value=None):
        self.value = value


class FileTypeError(Exception):
    def __init__(self, value=None):
        self.value = value


class MissingFFTError(Exception):
    def __init__(self, value=None):
        self.value = value


class LoginException(Exception):
    pass


class MissingTagException(Exception):
    pass


def get_remote_file_size(ftp_connector, filename, storage):

        def dir_callback(val):
            storage.append(val.split()[4])
        ftp_connector.dir(filename, dir_callback)


def check_pkgs_integrity(filelist, logger, ftp_connector,
                         timeout=120, sleep_time=10):
    """
    Checks if files are not being uploaded to server.
    @timeout - time after which the script will register an error.
    """
    ref_1 = []
    ref_2 = []
    i = 1
    print >> sys.stdout, "\nChecking packages integrity."
    for filename in filelist:
        # ref_1.append(self.get_remote_file_size(filename))
        get_remote_file_size(ftp_connector, filename, ref_1)
    print >> sys.stdout, "\nGoing to sleep for %i sec." % (sleep_time,)
    time.sleep(sleep_time)

    while sleep_time*i < timeout:
        for filename in filelist:
            # ref_2.append(self.get_remote_file_size(filename))
            get_remote_file_size(ftp_connector, filename, ref_2)
        if ref_1 == ref_2:
            print >> sys.stdout, "\nIntegrity OK:)"
            logger.info("Packages integrity OK.")
            break
        else:
            print >> sys.stdout, "\nWaiting %d time for itegrity..." % (i,)
            logger.info("\nWaiting %d time for itegrity..." % (i,))
            i += 1
            ref_1, ref_2 = ref_2, []
            time.sleep(sleep_time)
    else:
        not_finished_files = []
        for count, val1 in enumerate(ref_1):
            if val1 != ref_2[count]:
                not_finished_files.append(filelist[count])

        print >> sys.stdout, "\nOMG, OMG something wrong with integrity."
        logger.error("Integrity check faild for files %s"
                     % (not_finished_files,))


def extract_package(package_name, path, logger):
    try:
        if ".tar" in package_name:
            TarFile.open(package_name).extractall(path)
        elif ".zip" in package_name:
            ZipFile(package_name).extractall(path)
        else:
            raise FileTypeError("It's not a TAR or ZIP archive.")
    except Exception as err:
        register_exception(alert_admin=True,
                           prefix="Elsevier error extracting package.")
        logger.error("Error extraction package file: %s %s"
                     % (path, err))



# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2015 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Contains conversion package from CDS to INSPIRE MARCXML.

>>> from harvestingkit.bibrecord import BibRecordPackage
>>> from harvestingkit.inspire_cds_package.from_cds import CDS2Inspire
>>> bibrecs = BibRecordPackage("cds.xml")
>>> bibrecs.parse()
>>> inspire_xml = CDS2Inspire.convert(bibrecs.get_records())
"""

from __future__ import print_function

import os
import re

from tempfile import mkstemp

from ..bibrecord import (record_get_field_instances,
                         record_add_field,
                         record_get_field_values,
                         record_delete_field,
                         record_delete_fields,
                         record_replace_field,
                         field_get_subfield_instances,
                         create_field,
                         field_swap_subfields,
                         field_get_subfields)

from ..utils import (
    punctuate_authorname,
    convert_images,
    convert_date_to_iso,
    unzip,
    locate,
    download_file,
)
from .base import MARCXMLConversion


class CDS2Inspire(MARCXMLConversion):

    """Convert CDS to INSPIRE."""

    # By setting the class variable here, we run it only once a session
    kbs = MARCXMLConversion.load_config("cds", "inspire")

    def __init__(self, bibrec, strip_fields_list=None):
        """Create."""
        super(CDS2Inspire, self).__init__(bibrec, strip_fields_list)
        self.collections = set([])

    def get_record(self):
        """Override the base get_record."""
        self.update_system_numbers()
        self.add_systemnumber("CDS")
        self.fields_list = [
            "024", "041", "035", "037", "088", "100",
            "110", "111", "242", "245", "246", "260",
            "269", "300", "502", "650", "653", "693",
            "700", "710", "773", "856", "520", "500",
            "980"
        ]
        self.keep_only_fields()

        self.determine_collections()
        self.add_cms_link()
        self.update_languages()
        self.update_reportnumbers()
        self.update_date()
        self.update_pagenumber()
        self.update_authors()
        self.update_subject_categories("SzGeCERN", "INSPIRE", "categories_inspire")
        self.update_keywords()
        self.update_experiments()
        self.update_collaboration()
        self.update_journals()
        self.update_links_and_ffts()

        if 'THESIS' in self.collections:
            self.update_thesis_supervisors()
            self.update_thesis_information()

        if 'NOTE' in self.collections:
            self.add_notes()

        for collection in self.collections:
            record_add_field(self.record,
                             tag='980',
                             subfields=[('a', collection)])
        self.remove_controlfields()
        return self.record

    def determine_collections(self):
        """Try to determine which collections this record should belong to."""
        for value in record_get_field_values(self.record, '980', code='a'):
            if 'NOTE' in value.upper():
                self.collections.add('NOTE')
            if 'THESIS' in value.upper():
                self.collections.add('THESIS')
            if 'CONFERENCEPAPER' in value.upper():
                self.collections.add('ConferencePaper')
            if "HIDDEN" in value.upper():
                self.hidden = True

        if self.is_published():
            self.collections.add("PUBLISHED")
            self.collections.add("CITEABLE")

        if 'NOTE' not in self.collections:
            from itertools import product
            # TODO: Move this to a KB
            kb = ['ATLAS-CONF-', 'CMS-PAS-', 'ATL-', 'CMS-DP-',
                  'ALICE-INT-', 'LHCb-PUB-']
            values = record_get_field_values(self.record, "088", code='a')
            for val, rep in product(values, kb):
                if val.startswith(rep):
                    self.collections.add('NOTE')
                    break

        # 980 Arxiv tag
        if record_get_field_values(self.record, '035',
                                   filter_subfield_code="a",
                                   filter_subfield_value="arXiv"):
            self.collections.add("arXiv")

        # 980 HEP && CORE
        self.collections.add('HEP')
        self.collections.add('CORE')

        # 980 Conference Note
        if 'ConferencePaper' not in self.collections:
            for value in record_get_field_values(self.record,
                                                 tag='962',
                                                 code='n'):
                if value[-2:].isdigit():
                    self.collections.add('ConferencePaper')
                    break
        # Clear out any existing ones.
        record_delete_fields(self.record, "980")

    def is_published(self):
        """Check fields 980 and 773 to see if the record has already been published.

        :return: True is published, else False
        """
        field980 = record_get_field_instances(self.record, '980')
        field773 = record_get_field_instances(self.record, '773')
        for f980 in field980:
            if 'a' in field_get_subfields(f980):
                for f773 in field773:
                    if 'p' in field_get_subfields(f773):
                        return True
        return False

    def add_cms_link(self):
        """Special handling if record is a CMS NOTE."""
        intnote = record_get_field_values(self.record, '690',
                                          filter_subfield_code="a",
                                          filter_subfield_value='INTNOTE')
        if intnote:
            val_088 = record_get_field_values(self.record,
                                              tag='088',
                                              filter_subfield_code="a")
            for val in val_088:
                if 'CMS' in val:
                    url = ('http://weblib.cern.ch/abstract?CERN-CMS' +
                           val.split('CMS', 1)[-1])
                    record_add_field(self.record,
                                     tag='856',
                                     ind1='4',
                                     subfields=[('u', url)])

    def update_system_numbers(self):
        """035 Externals."""
        scn_035_fields = record_get_field_instances(self.record, '035')
        forbidden_values = ["cercer",
                            "inspire",
                            "xx",
                            "cern annual report",
                            "cmscms",
                            "wai01"]
        for field in scn_035_fields:
            subs = field_get_subfields(field)
            if '9' in subs:
                if 'a' not in subs:
                    continue
                for sub in subs['9']:
                    if sub.lower() in forbidden_values:
                        break
                else:
                    # No forbidden values (We did not "break")
                    suffixes = [s.lower() for s in subs['9']]
                    if 'spires' in suffixes:
                        new_subs = [('a', 'SPIRES-%s' % subs['a'][0])]
                        record_add_field(
                            self.record, '970', subfields=new_subs)
                        continue
            if 'a' in subs:
                for sub in subs['a']:
                    if sub.lower() in forbidden_values:
                        record_delete_field(self.record, tag="035",
                                            field_position_global=field[4])

    def update_reportnumbers(self):
        """Handle reportnumbers. """
        rep_088_fields = record_get_field_instances(self.record, '088')
        for field in rep_088_fields:
            subs = field_get_subfields(field)
            if '9' in subs:
                for val in subs['9']:
                    if val.startswith('P0') or val.startswith('CM-P0'):
                        sf = [('9', 'CERN'), ('b', val)]
                        record_add_field(self.record, '595', subfields=sf)
            for key, val in field[0]:
                if key in ['a', '9'] and not val.startswith('SIS-'):
                    record_add_field(
                        self.record, '037', subfields=[('a', val)])
        record_delete_fields(self.record, "088")

        # 037 Externals also...
        rep_037_fields = record_get_field_instances(self.record, '037')
        for field in rep_037_fields:
            subs = field_get_subfields(field)
            if 'a' in subs:
                for value in subs['a']:
                    if 'arXiv' in value:
                        new_subs = [('a', value), ('9', 'arXiv')]
                        for fld in record_get_field_instances(self.record,  '695'):
                            for key, val in field_get_subfield_instances(fld):
                                if key == 'a':
                                    new_subs.append(('c', val))
                                    break
                        nf = create_field(subfields=new_subs)
                        record_replace_field(self.record, '037', nf, field[4])
            for key, val in field[0]:
                if key in ['a', '9'] and val.startswith('SIS-'):
                    record_delete_field(
                        self.record, '037', field_position_global=field[4])

    def update_date(self):
        """269 Date normalization."""
        for field in record_get_field_instances(self.record, '269'):
            for idx, (key, value) in enumerate(field[0]):
                if key == "c":
                    field[0][idx] = ("c", convert_date_to_iso(value))
                    record_delete_fields(self.record, "260")

        if 'THESIS' not in self.collections:
            for field in record_get_field_instances(self.record, '260'):
                record_add_field(self.record, '269', subfields=field[0])
            record_delete_fields(self.record, '260')

    def update_pagenumber(self):
        """300 page number."""
        for field in record_get_field_instances(self.record, '300'):
            for idx, (key, value) in enumerate(field[0]):
                if key == 'a':
                    if "mult." not in value and value != " p":
                        field[0][idx] = ('a', re.sub(r'[^\d-]+', '', value))
                    else:
                        record_delete_field(self.record, '300',
                                            field_position_global=field[4])
                        break

    def update_authors(self):
        """100 & 700 punctuate author names."""
        author_names = record_get_field_instances(self.record, '100')
        author_names.extend(record_get_field_instances(self.record, '700'))
        for field in author_names:
            subs = field_get_subfields(field)
            if 'i' not in subs or 'XX' in subs['i']:
                if 'j' not in subs or 'YY' in subs['j']:
                    for idx, (key, value) in enumerate(field[0]):
                        if key == 'a':
                            field[0][idx] = ('a', punctuate_authorname(value))

    def update_thesis_supervisors(self):
        """700 -> 701 Thesis supervisors."""
        for field in record_get_field_instances(self.record, '700'):
            record_add_field(self.record, '701', subfields=field[0])
        record_delete_fields(self.record, '700')

    def update_thesis_information(self):
        """501 degree info - move subfields."""
        fields_501 = record_get_field_instances(self.record, '502')
        for idx, field in enumerate(fields_501):
            new_subs = []
            for key, value in field[0]:
                if key == 'a':
                    new_subs.append(('b', value))
                elif key == 'b':
                    new_subs.append(('c', value))
                elif key == 'c':
                    new_subs.append(('d', value))
                else:
                    new_subs.append((key, value))
            fields_501[idx] = field_swap_subfields(field, new_subs)

    def update_keywords(self):
        """653 Free Keywords."""
        for field in record_get_field_instances(self.record, '653', ind1='1'):
            subs = field_get_subfields(field)
            new_subs = []
            if 'a' in subs:
                for val in subs['a']:
                    new_subs.extend([('9', 'author'), ('a', val)])
            new_field = create_field(subfields=new_subs, ind1='1')
            record_replace_field(
                self.record, '653', new_field, field_position_global=field[4])

    def update_experiments(self):
        """Experiment mapping."""
        # 693 Remove if 'not applicable'
        for field in record_get_field_instances(self.record, '693'):
            subs = field_get_subfields(field)
            all_subs = subs.get('a', []) + subs.get('e', [])
            if 'not applicable' in [x.lower() for x in all_subs]:
                record_delete_field(self.record, '693',
                                    field_position_global=field[4])
            new_subs = []
            experiment_a = ""
            experiment_e = ""
            for (key, value) in subs.iteritems():
                if key == 'a':
                    experiment_a = value[0]
                    new_subs.append((key, value[0]))
                elif key == 'e':
                    experiment_e = value[0]
            experiment = "%s---%s" % (experiment_a.replace(" ", "-"),
                                      experiment_e)
            translated_experiments = self.get_config_item(experiment,
                                                          "experiments")
            new_subs.append(("e", translated_experiments))
            record_delete_field(self.record, tag="693",
                                field_position_global=field[4])
            record_add_field(self.record, "693", subfields=new_subs)

    def update_collaboration(self):
        """710 Collaboration."""
        for field in record_get_field_instances(self.record, '710'):
            subs = field_get_subfield_instances(field)
            for idx, (key, value) in enumerate(subs[:]):
                if key == '5':
                    subs.pop(idx)
                elif value.startswith('CERN. Geneva'):
                    subs.pop(idx)
            if len(subs) == 0:
                record_delete_field(self.record,
                                    tag='710',
                                    field_position_global=field[4])

    def update_journals(self):
        """773 journal translations."""
        for field in record_get_field_instances(self.record, '773'):
            subs = field_get_subfield_instances(field)
            new_subs = []
            for idx, (key, value) in enumerate(subs):
                if key == 'p':
                    new_subs.append((key, self.get_config_item(value,
                                                               "journals")))
                else:
                    new_subs.append((key, value))
            record_delete_field(self.record, tag="773",
                                field_position_global=field[4])
            record_add_field(self.record, "773", subfields=new_subs)

    def update_links_and_ffts(self):
        """FFT (856) Dealing with graphs."""
        figure_counter = 0
        for field in record_get_field_instances(self.record,
                                                tag='856',
                                                ind1='4'):
            subs = field_get_subfields(field)

            newsubs = []
            remove = False

            if 'z' in subs:
                is_figure = [s for s in subs['z'] if "figure" in s.lower()]
                if is_figure and 'u' in subs:
                    is_subformat = [
                        s for s in subs['u'] if "subformat" in s.lower()]
                    if not is_subformat:
                        url = subs['u'][0]
                        if url.endswith(".pdf"):
                            # We try to convert
                            fd, local_url = mkstemp(suffix=os.path.basename(url))
                            os.close(fd)
                            self.logger.info(
                                "Downloading %s into %s" % (url, local_url))
                            plotfile = ""
                            try:
                                plotfile = download_file(url=url,
                                                         download_to_file=local_url)
                            except Exception as e:
                                self.logger.exception(e)
                                remove = True
                            if plotfile:
                                converted = convert_images([plotfile])
                                if converted:
                                    url = converted.pop()
                                    msg = "Successfully converted %s to %s" \
                                          % (local_url, url)
                                    self.logger.info(msg)
                                else:
                                    msg = "Conversion failed on %s" \
                                          % (local_url,)
                                    self.logger.error(msg)
                                    url = None
                                    remove = True
                        if url:
                            newsubs.append(('a', url))
                            newsubs.append(('t', 'Plot'))
                            figure_counter += 1
                            if 'y' in subs:
                                newsubs.append(
                                    ('d', "%05d %s" % (figure_counter, subs['y'][0])))
                                newsubs.append(('n', subs['y'][0]))
                            else:
                                # Get basename without extension.
                                name = os.path.basename(
                                    os.path.splitext(subs['u'][0])[0])
                                newsubs.append(
                                    ('d', "%05d %s" % (figure_counter, name)))
                                newsubs.append(('n', name))

            if not newsubs and 'u' in subs:
                is_fulltext = [s for s in subs['u'] if ".pdf" in s]
                if is_fulltext:
                    newsubs = [('t', 'INSPIRE-PUBLIC'), ('a', subs['u'][0])]

            if not newsubs and 'u' in subs:
                remove = True
                is_zipfile = [s for s in subs['u'] if ".zip" in s]
                if is_zipfile:
                    url = is_zipfile[0]

                    local_url = os.path.join(self.get_local_folder(), os.path.basename(url))
                    self.logger.info("Downloading %s into %s" %
                                     (url, local_url))
                    zipped_archive = ""
                    try:
                        zipped_archive = download_file(url=is_zipfile[0],
                                                      download_to_file=local_url)
                    except Exception as e:
                        self.logger.exception(e)
                        remove = True
                    if zipped_archive:
                        unzipped_archive = unzip(zipped_archive)
                        list_of_pngs = locate("*.png", unzipped_archive)
                        for png in list_of_pngs:
                            if "_vti_" in png or "__MACOSX" in png:
                                continue
                            figure_counter += 1
                            plotsubs = []
                            plotsubs.append(('a', png))
                            caption = '%05d %s' % (
                                figure_counter, os.path.basename(png))
                            plotsubs.append(('d', caption))
                            plotsubs.append(('t', 'Plot'))
                            record_add_field(
                                self.record, 'FFT', subfields=plotsubs)

            if not remove and not newsubs and 'u' in subs:
                urls = ('http://cdsweb.cern.ch', 'http://cms.cern.ch',
                        'http://cmsdoc.cern.ch', 'http://documents.cern.ch',
                        'http://preprints.cern.ch', 'http://cds.cern.ch')
                for val in subs['u']:
                    if any(url in val for url in urls):
                        remove = True
                        break
                    if val.endswith('ps.gz'):
                        remove = True

            if newsubs:
                record_add_field(self.record, 'FFT', subfields=newsubs)
                remove = True

            if remove:
                record_delete_field(self.record, '856', ind1='4',
                                    field_position_global=field[4])

    def add_notes(self):
        """500 - Preliminary results."""
        subs = [('a', "Preliminary results")]
        record_add_field(self.record, "500", subfields=subs)



# -*- coding: utf-8 -*-
##
## This file is part of Harvesting Kit.
## Copyright (C) 2014 CERN.
##
## Harvesting Kit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2 of the
## License, or (at your option) any later version.
##
## Harvesting Kit is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
## 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from __future__ import print_function

import sys
import requests

from bs4 import BeautifulSoup
from urlparse import urlparse
from os.path import join
from os import makedirs
from re import sub

from harvestingkit.minidom_utils import (get_value_in_tag,
                                         xml_to_text)
from harvestingkit.utils import (collapse_initials,
                                 fix_journal_name,
                                 download_file)
from harvestingkit.bibrecord import (
    record_add_field,
    create_record,
    record_xml_output,
)

from xml.dom.minidom import (parse,
                             parseString)
from harvestingkit.jats_package import JatsPackage

try:
    from invenio.config import CFG_EDPSCIENCE_OUT_FOLDER
except ImportError:
    from distutils.sysconfig import get_python_lib

    CFG_EDPSCIENCE_OUT_FOLDER = join(get_python_lib(),
                                     "harvestingkit",
                                     "edpsciences")


class EDPSciencesPackage(JatsPackage):
    """
    This class is specialized in parsing files from EdpSciences FTP server
    and converting them in Marc XML.
    """
    def __init__(self, journal_mappings={}):
        super(EDPSciencesPackage, self).__init__(journal_mappings)

    def _get_references(self):
        for ref in self.document.getElementsByTagName('ref'):
            label = ref.getAttribute('id')
            label = sub(r'\D', '', label)
            text_ref = ''
            ext_link = ''
            for mixed in ref.getElementsByTagName('mixed-citation'):
                ref_type = mixed.getAttribute('publication-type')
                if ref_type == 'thesis':
                    text_ref = get_value_in_tag(ref, 'mixed-citation')
                elif ref_type == 'conf-proc':
                    text_ref = get_value_in_tag(ref, 'mixed-citation')
                elif ref_type == 'other' or ref_type == 'web':
                    text_ref = get_value_in_tag(ref, 'mixed-citation')
                    ext_link = get_value_in_tag(mixed, 'ext-link')
                elif ref_type == 'book':
                    text_ref = xml_to_text(mixed)
            authors = []
            for auth in ref.getElementsByTagName('string-name'):
                surname = get_value_in_tag(auth, 'surname')
                given_names = get_value_in_tag(auth, 'given-names')
                given_names = collapse_initials(given_names)
                authors.append('%s, %s' % (surname, given_names))
            year = get_value_in_tag(ref, 'year')
            source = get_value_in_tag(ref, 'source')
            volume = get_value_in_tag(ref, 'volume')
            page = get_value_in_tag(ref, 'fpage')
            if ref_type == 'journal':
                source, vol = fix_journal_name(source, self.journal_mappings)
                if vol:
                    volume = vol + volume
            yield (label, ref_type, text_ref, ext_link,
                   authors, year, source, volume, page)

    def _add_references(self, rec, ref_extract_callback=None):
        for label, ref_type, text_ref, ext_link, authors, year, \
                source, volume, page in self._get_references():
            subfields = []
            if label:
                subfields.append(('o', label))
            if text_ref:
                if ref_extract_callback:
                    ref_xml = ref_extract_callback(text_ref)
                    dom = parseString(ref_xml)
                    fields = dom.getElementsByTagName("datafield")[0]
                    fields = fields.getElementsByTagName("subfield")
                    for field in fields:
                        data = field.firstChild.data
                        code = field.getAttribute("code")
                        subfields.append((code, data))
                    if fields:
                        subfields.append(('9', 'refextract'))
                else:
                    subfields.append(('m', text_ref))
            if ref_type:
                subfields.append(('d', ref_type))
            if ext_link:
                subfields.append(('u', ext_link))
            for author in authors:
                subfields.append(('h', author))
            if year:
                subfields.append(('y', year))
            if source and volume and page:
                subfields.append(('s', source + "," + volume + "," + page))
            elif source and volume:
                subfields.append(('s', source + "," + volume))
            elif source and page:
                subfields.append(('s', source + "," + page))
            elif source:
                subfields.append(('s', source))
            record_add_field(rec, '999', ind1='C', ind2='5',
                             subfields=subfields)

    def _get_note(self, note_id):
        for tag in self.document.getElementsByTagName('fn'):
            if tag.getAttribute('id') == note_id:
                for label in tag.getElementsByTagName('label'):
                    tag.removeChild(label)
                return xml_to_text(tag)

    def get_date(self, filename):
        self.document = parse(filename)
        return self._get_date()

    def _format_abstract(self, abstract):
        abstract = abstract.replace("Context.", "Context:<br/>")
        abstract = abstract.replace("Aims.", "<br/>Aims:<br/>")
        abstract = abstract.replace("Methods.", "<br/>Methods:<br/>")
        abstract = abstract.replace("Results.", "<br/>Results:<br/>")
        abstract = abstract.replace("Conclusions..", "<br/>Conclusions:<br/>")
        return abstract

    def get_record(self, fileName, ref_extract_callback=None):
        """
        Gets the Marc xml of the files in xaml_jp directory

        :param fileName: the name of the file to parse.
        :type fileName: string
        :param refextract_callback: callback to be used to extract
                                    unstructured references. It should
                                    return a marcxml formated string
                                    of the reference.
        :type refextract_callback: callable

        :returns: a string with the marc xml version of the file.
        """
        self.document = parse(fileName)
        article_type = self._get_article_type()
        if article_type not in ['research-article',
                                'introduction',
                                'letter']:
            return ''
        rec = create_record()
        title, subtitle, notes = self._get_title()
        subfields = []
        if subtitle:
            subfields.append(('b', subtitle))
        if title:
            subfields.append(('a', title))
            record_add_field(rec, '245', subfields=subfields)
        subjects = self.document.getElementsByTagName('kwd')
        subjects = map(xml_to_text, subjects)
        for note_id in notes:
            note = self._get_note(note_id)
            if note:
                record_add_field(rec, '500', subfields=[('a', note)])
        for subject in subjects:
            record_add_field(rec, '650', ind1='1', ind2='7',
                             subfields=[('2', 'EDPSciences'),
                                        ('a', subject)])
        keywords = self._get_keywords()
        for keyword in keywords:
            record_add_field(rec, '653', ind1='1', subfields=[('a', keyword),
                                                              ('9', 'author')])
        journal, volume, issue, year, date, doi, page,\
            fpage, lpage = self._get_publication_information()
        astronomy_journals = ['EAS Publ.Ser.', 'Astron.Astrophys.']
        if journal in astronomy_journals:
            record_add_field(rec, '650', ind1='1', ind2='7',
                             subfields=[('2', 'INSPIRE'),
                                        ('a', 'Astrophysics')])
        if date:
            record_add_field(rec, '260', subfields=[('c', date),
                                                    ('t', 'published')])
        if doi:
            record_add_field(rec, '024', ind1='7', subfields=[('a', doi),
                                                              ('2', 'DOI')])
        abstract = self._get_abstract()
        abstract = self._format_abstract(abstract)
        if abstract:
            record_add_field(rec, '520', subfields=[('a', abstract),
                                                    ('9', 'EDPSciences')])
        license, license_type, license_url = self._get_license()
        subfields = []
        if license:
            subfields.append(('a', license))
        if license_url:
            subfields.append(('u', license_url))
        if subfields:
            record_add_field(rec, '540', subfields=subfields)
        if license_type == 'open-access':
            self._attach_fulltext(rec, doi)
        number_of_pages = self._get_page_count()
        if number_of_pages:
            record_add_field(rec, '300', subfields=[('a', number_of_pages)])
        c_holder, c_year, c_statement = self._get_copyright()
        if c_holder and c_year:
            record_add_field(rec, '542', subfields=[('d', c_holder),
                                                    ('g', c_year),
                                                    ('e', 'Article')])
        elif c_statement:
            record_add_field(rec, '542', subfields=[('f', c_statement),
                                                    ('e', 'Article')])
        subfields = []
        if journal:
            subfields.append(('p', journal))
        if issue:
            subfields.append(('n', issue))
        if volume:
            subfields.append(('v', volume))
        if fpage and lpage:
            subfields.append(('c', '%s-%s' % (fpage,
                                              lpage)))
        elif page:
            subfields.append(('c', page))
        if year:
            subfields.append(('y', year))
        record_add_field(rec, '773', subfields=subfields)
        record_add_field(rec, '980', subfields=[('a', 'HEP')])
        conference = ''
        for tag in self.document.getElementsByTagName('conference'):
            conference = xml_to_text(tag)
        if conference:
            record_add_field(rec, '980', subfields=[('a', 'ConferencePaper')])
            record_add_field(rec, '500', subfields=[('a', conference)])
        self._add_references(rec, ref_extract_callback)
        self._add_authors(rec)
        try:
            return record_xml_output(rec)
        except UnicodeDecodeError:
            message = "Found a bad char in the file for the article " + doi
            sys.stderr.write(message)
            return ""

    def get_record_rich(self, filename, ref_extract_callback=None):
        """
        Gets the Marc xml of the files in xaml_rich directory

        :param fileName: the name of the file to parse.
        :type fileName: string

        :returns: a string with the marc xml version of the file.
        """
        self.document = parse(filename)
        rec = create_record()
        articles = self.document.getElementsByTagName('ArticleID')
        for article in articles:
            article_type = article.getAttribute('Type')
            if not article_type == 'Article':
                return ''
            doi = get_value_in_tag(self.document, 'DOI')
            date = ''
            for tag in self.document.getElementsByTagName('Accepted'):
                year = get_value_in_tag(tag, 'Year')
                month = get_value_in_tag(tag, 'Month').zfill(2)
                day = get_value_in_tag(tag, 'Day').zfill(2)
                date = "%s-%s-%s" % (year, month, day)
            if not date:
                for tag in self.document.getElementsByTagName('OnlineDate'):
                    year = get_value_in_tag(tag, 'Year')
                    month = get_value_in_tag(tag, 'Month').zfill(2)
                    day = get_value_in_tag(tag, 'Day').zfill(2)
                    date = "%s-%s-%s" % (year, month, day)
            first_page = get_value_in_tag(article, 'FirstPage')
            last_page = get_value_in_tag(article, 'LastPage')
            subjects = article.getElementsByTagName('Keyword')
            subjects = map(xml_to_text, subjects)
            subject = ', '.join(subjects)
            copyright_statement = get_value_in_tag(article, 'Copyright')
        journal = get_value_in_tag(self.document, 'JournalTitle')
        journal, volume = fix_journal_name(journal, self.journal_mappings)
        issues = self.document.getElementsByTagName('IssueID')
        for issue in issues:
            volume += get_value_in_tag(issue, 'Volume')
            year = get_value_in_tag(issue, 'Year')
        title = get_value_in_tag(self.document, 'Title')
        authors = self.document.getElementsByTagName('Author')
        affiliations = self.document.getElementsByTagName('Affiliation')

        def affiliation_pair(a):
            return a.getAttribute('ID'), get_value_in_tag(
                a, 'UnstructuredAffiliation'
            )

        affiliations = map(affiliation_pair, affiliations)
        affiliations = dict(affiliations)

        def author_pair(a):
            surname = get_value_in_tag(a, 'LastName')
            first_name = get_value_in_tag(a, 'FirstName')
            middle_name = get_value_in_tag(a, 'MiddleName')
            if middle_name:
                name = '%s, %s %s' % (surname, first_name, middle_name)
            else:
                name = '%s, %s' % (surname, first_name)
            try:
                affid = a.getElementsByTagName(
                    'AffiliationID'
                )[0].getAttribute('Label')
                affiliation = affiliations[affid]
            except IndexError:
                affiliation = ''
            except KeyError:
                affiliation = ''
            return name, affiliation

        authors = map(author_pair, authors)
        abstract = get_value_in_tag(self.document, 'Abstract')
        references = self.document.getElementsByTagName('Bibliomixed')

        for reference in references:
            subfields = []
            label = reference.getAttribute('N')
            if label:
                subfields.append(('o', label))
            bibliosets = reference.getElementsByTagName('Biblioset')
            for tag in bibliosets:
                ref_year = get_value_in_tag(tag, 'Date')
                ref_journal = get_value_in_tag(tag, 'JournalShortTitle')
                ref_journal, ref_volume = fix_journal_name(
                    ref_journal, self.journal_mappings
                )
                ref_volume += get_value_in_tag(tag, 'Volume')
                ref_page = get_value_in_tag(tag, 'ArtPageNums')
                if ref_year:
                    subfields.append(('y', ref_year))
                if ref_journal and ref_volume and ref_page:
                    subfields.append(('s', '%s,%s,%s' % (ref_journal,
                                                         ref_volume,
                                                         ref_page)))
                reference.removeChild(tag)
            text_ref = xml_to_text(reference)
            if ref_extract_callback:
                ref_xml = ref_extract_callback(text_ref)
                dom = parseString(ref_xml)
                fields = dom.getElementsByTagName("datafield")[0]
                fields = fields.getElementsByTagName("subfield")
                if fields:
                    subfields.append(('9', 'refextract'))
                for field in fields:
                    data = field.firstChild.data
                    code = field.getAttribute("code")
                    if code == 'm' and bibliosets:
                        continue
                    else:
                        subfields.append((code, data))
            else:
                subfields.append(('m', text_ref))
            if subfields:
                record_add_field(rec, '999', ind1='C', ind2='5',
                                 subfields=subfields)

        if title:
            record_add_field(rec, '245', subfields=[('a', title)])
        if date:
            record_add_field(rec, '260', subfields=[('c', date),
                                                    ('t', 'published')])
        if doi:
            record_add_field(rec, '024', ind1='7', subfields=[('a', doi),
                                                              ('2', 'DOI')])
        if abstract:
            record_add_field(rec, '520', subfields=[('a', abstract),
                                                    ('9', 'EDPSciences')])
        first_author = True
        for author in authors:
            if first_author:
                subfields = [('a', author[0])]
                if author[1]:
                    subfields.append(('v', author[1]))
                record_add_field(rec, '100', subfields=subfields)
                first_author = False
            else:
                subfields = [('a', author[0])]
                if author[1]:
                    subfields.append(('v', author[1]))
                record_add_field(rec, '700', subfields=subfields)
        subfields = []
        if journal and volume and first_page:
            subfields.append(('s', "%s,%s,%s" % (journal,
                                                 volume,
                                                 first_page)))
        if first_page and last_page:
            try:
                nuber_of_pages = int(last_page) - int(first_page)
                record_add_field(rec, '300',
                                 subfields=[('a', str(nuber_of_pages))])
            except ValueError:
                pass
            subfields.append(('c', '%s-%s' % (first_page,
                                              last_page)))
        if year:
            subfields.append(('y', year))
        record_add_field(rec, '773', subfields=subfields)
        record_add_field(rec, '980', subfields=[('a', 'HEP')])
        if copyright_statement:
            record_add_field(rec, '542',
                             subfields=[('f', copyright_statement)])
        if subject:
            record_add_field(rec, '650', ind1='1', ind2='7',
                             subfields=[('2', 'EDPSciences'),
                                        ('a', subject)])
        try:
            return record_xml_output(rec)
        except UnicodeDecodeError:
            message = "Found a bad char in the file for the article " + doi
            sys.stderr.write(message)
            return ""

    def _attach_fulltext(self, rec, doi):
        url = 'http://dx.doi.org/' + doi
        page = requests.get(url)
        #url after redirect
        url = page.url
        page = page.text
        parsed_uri = urlparse(url)
        domain = '{uri.scheme}://{uri.netloc}'.format(uri=parsed_uri)
        page = BeautifulSoup(page)
        try:
            if 'epjconf' in doi:
                div = page.body.find('div', attrs={'id': 'header'})
            else:
                div = page.body.find('div', attrs={
                    'class': 'module_background files'
                })
            links = div.findAll('a')
        except AttributeError:
            return
        for pdf in links:
            if pdf['href'].endswith('pdf'):
                link_to_pdf = domain + pdf['href']
                record_add_field(rec, '856', ind1='4',
                                 subfields=[('u', link_to_pdf),
                                            ('y', 'EDP Sciences server')])
                out_folder = join(CFG_EDPSCIENCE_OUT_FOLDER,
                                  "fulltexts")
                try:
                    makedirs(out_folder)
                    filename = join(out_folder,
                                    link_to_pdf.split('/')[-1])
                except (IOError, OSError):
                    # Problem creating folder
                    filename = None

                filename = download_file(from_url=link_to_pdf,
                                         to_filename=filename,
                                         retry_count=5)
                record_add_field(rec, 'FFT',
                                 subfields=[('a', filename),
                                            ('t', 'INSPIRE-PUBLIC'),
                                            ('d', 'Fulltext')])


if __name__ == '__main__':
    filename = sys.argv[1]
    edp = EDPSciencesPackage()
    if 'xml_rich' in filename:
        print(edp.get_record_rich(filename))
    else:
        print(edp.get_record(filename))



# -*- coding: utf-8 -*-
##
## This file is part of Harvesting Kit.
## Copyright (C) 2013, 2014 CERN.
##
## Harvesting Kit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2 of the
## License, or (at your option) any later version.
##
## Harvesting Kit is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
## 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.


def contrast_out_cmp(x, y):
    def get_val_type(x):
        val = x
        val_vtex = False
        val_type = None

        if "vtex" in val:
            val_vtex = True
            val = val.split('_')[0]
            val = val.strip("vtex")
        else:
            val = val[4:]
            for w in ["P", "S", "Q", "AB", "J", "R"]:
                if w in val:
                    if w is "P":
                        val_type = 1
                    if w is "Q":
                        val_type = 2
                    if w in ["S", "R"]:
                        val_type = 3
                    if w is "AB":
                        val_type = 4
                    if w is "J":
                        val_type = 5
                val = val.strip(w)

        return val, val_type, val_vtex

    x_number, x_type, x_vtex = get_val_type(x)
    y_number, y_type, y_vtex = get_val_type(y)

    if x_vtex or y_vtex:
        if x_vtex and not y_vtex:
            return 1
        elif not x_vtex and y_vtex:
            return -1
        elif x_vtex and y_vtex:
            if int(x_number) > int(y_number):
                return 1
            elif int(x_number) < int(y_number):
                return -1
            else:
                return 0
    else:
        if int(x_number) > int(y_number):
            return 1
        elif int(x_number) < int(y_number):
            return -1
        else:
            if int(x_type) > int(y_type):
                return 1
            elif int(x_type) < int(y_type):
                return -1
            else:
                return 0


def find_package_name(path):
    try:
        return [p_name for p_name in path.split('/') if "CERN" in p_name or "vtex" in p_name][0]
    except:
        return "unknown"



# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2014, 2015 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from __future__ import print_function

import os
import sys
import datetime

from xml.dom.minidom import parse

from harvestingkit.minidom_utils import (get_value_in_tag,
                                         xml_to_text)
from harvestingkit.utils import (
    collapse_initials,
    fix_title_capitalization,
    convert_html_subscripts_to_latex,
    safe_title
)
from harvestingkit.bibrecord import (
    record_add_field,
    create_record,
    record_xml_output,
)
from harvestingkit.jats_package import JatsPackage


class DateNotFoundException(Exception):

    """Raised when publication date is not found in the document."""


class WorldScientific(JatsPackage):

    """Parsing articles from World Scientific converting them to MARCXML."""

    def __init__(self, journal_mappings={}):
        """Create instance of WorldScientific package."""
        self.url_prefix = "http://www.worldscientific.com/doi/pdf"
        super(WorldScientific, self).__init__(journal_mappings)

    def _get_date(self):
        def _extract_date(date):
            year = get_value_in_tag(date, 'year')
            month = get_value_in_tag(date, 'month').zfill(2)
            month = month if month != '00' else '01'
            day = get_value_in_tag(date, 'day').zfill(2)
            day = day if day != '00' else '01'
            return '%s-%s-%s' % (year, month, day)

        for date in self.document.getElementsByTagName('date'):
            if date.getAttribute('date-type') == 'published':
                return _extract_date(date)
        for date in self.document.getElementsByTagName('pub-date'):
            if date.getAttribute('pub-type') == 'ppub':
                return _extract_date(date)
        for date in self.document.getElementsByTagName('pub-date'):
            if date.getAttribute('pub-type') == 'epub':
                return _extract_date(date)
        for date in self.document.getElementsByTagName('pub-date'):
            if not date.getAttribute('pub-type'):
                return _extract_date(date)

        # Worst case, lets just return today
        raise DateNotFoundException

    def get_date(self, filename):
        """Return the date of the article in file."""
        try:
            self.document = parse(filename)
            return self._get_date()
        except DateNotFoundException:
            print("Date problem found in {0}".format(filename))
            return datetime.datetime.strftime(datetime.datetime.now(),
                                              "%Y-%m-%d")

    def _get_authors(self):
        authors = []
        for contrib in self.document.getElementsByTagName('contrib'):
            if contrib.getAttribute('contrib-type') == 'author':
                surname = get_value_in_tag(contrib, 'surname')
                given_names = get_value_in_tag(contrib, 'given-names')
                given_names = collapse_initials(given_names)
                name = '%s, %s' % (surname, given_names)
                name = safe_title(name)
                affiliations = []
                for aff in contrib.getElementsByTagName('aff'):
                    affiliations.append(xml_to_text(aff))
                emails = []
                for email in contrib.getElementsByTagName('email'):
                    emails.append(xml_to_text(email))
                collaborations = []
                for collaboration in contrib.getElementsByTagName("collab"):
                    collaborations.append(xml_to_text(collaboration))
                authors.append((name, affiliations, emails, collaborations))
        return authors

    def _add_authors(self, rec):
        authors = self._get_authors()
        first_author = True
        collaboration_added = False
        for author in authors:
            subfields = [('a', author[0])]
            if author[1]:
                for aff in author[1]:
                    subfields.append(('v', aff))
            if author[2]:
                for email in author[2]:
                    subfields.append(('m', email))
            if first_author:
                record_add_field(rec, '100', subfields=subfields)
                first_author = False
            else:
                record_add_field(rec, '700', subfields=subfields)
            if author[3] and not collaboration_added:
                collaborations = []
                for collab in author[3]:
                    collab_stripped = collab.replace("for the", "").strip()
                    if collab_stripped not in collaborations:
                        collaborations.append(collab_stripped)
                        record_add_field(rec, '710', subfields=[("g", collab_stripped)])
                collaboration_added = True

    def _get_related_article(self):
        for tag in self.document.getElementsByTagName('related-article'):
            if tag.getAttribute('ext-link-type') == 'doi':
                attributes = tag.attributes.keysNS()
                for attribute in attributes:
                    if attribute[1] == 'href':
                        return tag.getAttributeNS(*attribute)
        return ''

    def get_collection(self, journal):
        """Return this articles' collection."""
        conference = ''
        for tag in self.document.getElementsByTagName('conference'):
            conference = xml_to_text(tag)
        if conference or journal == "International Journal of Modern Physics: Conference Series":
            return [('a', 'HEP'), ('a', 'ConferencePaper')]
        elif self._get_article_type() == "review-article":
            return [('a', 'HEP'), ('a', 'Review')]
        else:
            return [('a', 'HEP'), ('a', 'Published')]

    def get_record(self, filename, ref_extract_callback=None):
        """Get the MARCXML of the files in xaml_jp directory.

        :param filename: the name of the file to parse.
        :type filename: string
        :param refextract_callback: callback to be used to extract
                                    unstructured references. It should
                                    return a marcxml formated string
                                    of the reference.
        :type refextract_callback: callable

        :returns: a string with the marc xml version of the file.
        """
        self.document = parse(filename)

        article_type = self._get_article_type()
        if article_type not in ['research-article',
                                'corrected-article',
                                'original-article',
                                'introduction',
                                'letter',
                                'correction',
                                'addendum',
                                'review-article',
                                'rapid-communications']:
            return ""

        rec = create_record()
        title, subtitle, notes = self._get_title()
        subfields = []
        if subtitle:
            subfields.append(('b', subtitle))
        if title:
            title = fix_title_capitalization(title)
            subfields.append(('a', title))
            record_add_field(rec, '245', subfields=subfields)
        for note_id in notes:
            note = self._get_note(note_id)
            if note:
                record_add_field(rec, '500', subfields=[('a', note)])
        keywords = self._get_keywords()
        for keyword in keywords:
            record_add_field(rec, '653', ind1='1', subfields=[('a', keyword),
                                                              ('9', 'author')])
        journal, volume, issue, year, date, doi, page,\
            fpage, lpage = self._get_publication_information()
        if date:
            record_add_field(rec, '260', subfields=[('c', date),
                                                    ('t', 'published')])
        if doi:
            record_add_field(rec, '024', ind1='7', subfields=[('a', doi),
                                                              ('2', 'DOI')])
        abstract = self._get_abstract()
        if abstract:
            abstract = convert_html_subscripts_to_latex(abstract)
            record_add_field(rec, '520', subfields=[('a', abstract),
                                                    ('9', 'World Scientific')])
        license, license_type, license_url = self._get_license()
        subfields = []
        if license:
            subfields.append(('a', license))
        if license_url:
            subfields.append(('u', license_url))
        if subfields:
            record_add_field(rec, '540', subfields=subfields)
        if license_type == 'open-access':
            self._attach_fulltext(rec, doi)
        number_of_pages = self._get_page_count()
        if number_of_pages:
            record_add_field(rec, '300', subfields=[('a', number_of_pages)])
        c_holder, c_year, c_statement = self._get_copyright()
        if c_holder and c_year:
            record_add_field(rec, '542', subfields=[('d', c_holder),
                                                    ('g', c_year),
                                                    ('e', 'Article')])
        elif c_statement:
            record_add_field(rec, '542', subfields=[('f', c_statement),
                                                    ('e', 'Article')])
        subfields = []
        if journal:
            subfields.append(('p', journal))
        if issue:
            subfields.append(('n', issue))
        if volume:
            subfields.append(('v', volume))
        if fpage and lpage:
            subfields.append(('c', '%s-%s' % (fpage,
                                              lpage)))
        elif page:
            subfields.append(('c', page))
        if year:
            subfields.append(('y', year))
        if article_type == 'correction':
            subfields.append(('m', 'Erratum'))
        elif article_type == 'addendum':
            subfields.append(('m', 'Addendum'))
        record_add_field(rec, '773', subfields=subfields)

        collections = self.get_collection(journal)
        for collection in collections:
            record_add_field(rec, '980', subfields=[collection])

        self._add_authors(rec)
        if article_type in ['correction',
                            'addendum']:
            related_article = self._get_related_article()
            if related_article:
                record_add_field(rec, '024', ind1='7', subfields=[('a', related_article),
                                                                  ('2', 'DOI')])
        try:
            return record_xml_output(rec)
        except UnicodeDecodeError:
            message = "Found a bad char in the file for the article " + doi
            sys.stderr.write(message)
            return ""

    def _attach_fulltext(self, rec, doi):
        """Attach fulltext FFT."""
        url = os.path.join(self.url_prefix, doi)
        record_add_field(rec, 'FFT',
                         subfields=[('a', url),
                                    ('t', 'INSPIRE-PUBLIC'),
                                    ('d', 'Fulltext')])
if __name__ == '__main__':
    filename = sys.argv[1]
    ws = WorldScientific()
    print(ws.get_record(filename))



# -*- coding: utf-8 -*-
##
## This file is part of Harvesting Kit.
## Copyright (C) 2014 CERN.
##
## Harvesting Kit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2 of the
## License, or (at your option) any later version.
##
## Harvesting Kit is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
## 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
import unittest
from xml.dom.minidom import parseString
from harvestingkit.minidom_utils import (get_inner_xml,
                                         xml_to_text,
                                         get_value_in_tag,
                                         get_attribute_in_tag)

sample_xml = "<Foo>"\
             "  some text"\
             "  <Bar name=\"a\">Bar A</Bar>"\
             "  <Bar name=\"b\">Bar B</Bar>"\
             "</Foo>"


class MinidomUtilsTests(unittest.TestCase):

    def setUp(self):
        self.document = parseString(sample_xml)

    def test_get_inner_xml(self):
        tag = self.document.getElementsByTagName('Foo')[0]
        inner_xml = "some text"\
                    " <Bar name=\"a\">Bar A</Bar>"\
                    " <Bar name=\"b\">Bar B</Bar>"
        self.assertEqual(get_inner_xml(tag), inner_xml)
        self.assertEqual(get_inner_xml(self.document), sample_xml)

    def test_xml_to_text(self):
        self.assertEqual(xml_to_text(self.document), "some text Bar A Bar B")
        self.assertEqual(xml_to_text(self.document, delimiter=""), "some textBar ABar B")
        self.assertEqual(xml_to_text(self.document, delimiter="", tag_to_remove="Bar"), "some text")

    def test_get_value_in_tag(self):
        self.assertEqual(get_value_in_tag(self.document, "Bar"), "Bar A")
        self.assertEqual(get_value_in_tag(self.document, "A"), "")

    def test_get_attribute_in_tag(self):
        self.assertEqual(get_attribute_in_tag(self.document, "Bar", "name"), ["a", "b"])
        self.assertEqual(get_attribute_in_tag(self.document, "Bar", "A"), [])
        self.assertEqual(get_attribute_in_tag(self.document, "A", "Bar"), [])


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(MinidomUtilsTests)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
##
## This file is part of Harvesting Kit.
## Copyright (C) 2013, 2014 CERN.
##
## Harvesting Kit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2 of the
## License, or (at your option) any later version.
##
## Harvesting Kit is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
## 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

import sys
import time
from datetime import datetime

from socket import timeout as socket_timeout_exception

from os import listdir
from os.path import (join,
                     walk)
from tempfile import mkdtemp
from xml.dom.minidom import parse

try:
    from contrast_out_config import *
except ImportError:
    pass


try:
    from invenio.config import CFG_TMPSHAREDDIR
except ImportError:
    from distutils.sysconfig import get_python_lib
    CFG_TMPSHAREDDIR = join(get_python_lib(),
                            "harvestingkit",
                            "tmp")

try:
    from invenio.config import CFG_CONTRASTOUT_DOWNLOADDIR
except ImportError:
    from distutils.sysconfig import get_python_lib
    CFG_CONTRASTOUT_DOWNLOADDIR = join(get_python_lib(),
                                       "harvestingkit",
                                       "var", "data",
                                       "scoap3", "elsevier")

try:
    from invenio.errorlib import register_exception
except ImportError:
    register_exception = lambda a=1, b=2: True

from .ftp_utils import FtpHandler
from .scoap3utils import (MD5Error,
                          NoNewFiles,
                          LoginException,
                          MissingTagException,
                          extract_package as scoap3utils_extract_package)
from .contrast_out_utils import (contrast_out_cmp,
                                 find_package_name)
from .minidom_utils import xml_to_text

from configparser import load_config

from .config import (CFG_CONFIG_PATH,
                     CFG_FTP_CONNECTION_ATTEMPTS,
                     CFG_FTP_TIMEOUT_SLEEP_DURATION)

CFG_READY_PACKAGES = join(CFG_CONTRASTOUT_DOWNLOADDIR, "ready_pkgs")
CFG_TAR_FILES = join(CFG_CONTRASTOUT_DOWNLOADDIR, "tar_files")


class ContrastOutConnector(object):
    def __init__(self, logger):
        self.ftp = None
        self.files_list = []
        self.retrieved_packages = {}
        self.retrieved_packages_unpacked = []
        self.path = None
        self.retrieved_packages_unpacked = []
        self.found_articles = []
        self.found_issues = []
        self.path_r_pkg = []
        self.logger = logger
        self.packages_delivery = []

        self.config = load_config(CFG_CONFIG_PATH, {'ELSEVIER': []})

    def connect(self):
        """Logs into the specified ftp server and returns connector."""
        for tried_connection_count in range(CFG_FTP_CONNECTION_ATTEMPTS):
            try:
                self.ftp = FtpHandler(self.config.ELSEVIER.URL,
                                      self.config.ELSEVIER.LOGIN,
                                      self.config.ELSEVIER.PASSWORD)
                self.logger.debug(('Successful connection to the '
                                   'Elsevier server'))
                return
            except socket_timeout_exception as err:
                self.logger.error(('Failed to connect %d of %d times. '
                                   'Will sleep for %d seconds and try again.')
                                  % (tried_connection_count+1,
                                     CFG_FTP_CONNECTION_ATTEMPTS,
                                     CFG_FTP_TIMEOUT_SLEEP_DURATION))
                time.sleep(CFG_FTP_TIMEOUT_SLEEP_DURATION)
            except Exception as err:
                self.logger.error(('Failed to connect to the '
                                   'Elsevier server. %s') % (err,))
                break

        raise LoginException(err)

    def _get_file_listing(self, phrase=None, new_only=True):
        if phrase:
            self.files_list = filter(lambda x: phrase in x, self.ftp.ls()[0])
        else:
            self.files_list = self.ftp.ls()[0]
        if new_only:
            self.files_list = (set(self.files_list)
                               - set(listdir(CFG_READY_PACKAGES)))

        return self.files_list

    def _download_file_listing(self):
        if self.files_list:
            # Prints stuff
            print("Downloading %i \".ready\" files." % (len(self.files_list)))
            # Create progrss bar
            total_count = len(self.files_list)

            for i, filename in enumerate(self.files_list, start=1):
                self.logger.info("Downloading %s of %s: %s"
                                 % (i, total_count, filename,))
                pkg_path = join(CFG_READY_PACKAGES, filename)
                self.path_r_pkg.append(pkg_path)
                try:
                    self.ftp.download(filename, CFG_READY_PACKAGES)
                except:
                    error_msg = "Error downloading file %s of %s: %s"
                    self.logger.error(error_msg % (i, total_count,
                                                   filename,))
                    print(sys.exc_info())
        else:
            self.logger.info("No new packages to download.")
            raise NoNewFiles

    def _get_packages(self):
        # Prints stuff
        print("Retrieving packages names.")
        # Create progrss bar
        total_count = len(self.files_list)

        for i, pack in enumerate(self.path_r_pkg, start=1):
            self.logger.info("Retrieved package name %s of %s: %s"
                             % (i, total_count, pack,))
            pack_xml = parse(pack)
            package_file = (pack_xml
                            .getElementsByTagName('dataset-package-file'))
            for pf in package_file:
                filename = pf.getElementsByTagName('filename')[0]
                md5_val = pf.getElementsByTagName('md5')[0]
                package_key = xml_to_text(filename)
                self.retrieved_packages[package_key] = xml_to_text(md5_val)

        return self.retrieved_packages

    def _download_tars(self, check_integrity=True):
        if check_integrity:
            self.ftp.check_pkgs_integrity(self.retrieved_packages, self.logger)

        print("Downloading %i tar packages." % (len(self.retrieved_packages)))
        # Create progrss bar
        total_count = len(self.files_list)

        for i, filename in enumerate(self.retrieved_packages.iterkeys(),
                                     start=1):
            self.logger.info("Downloading tar package %s of %s: %s"
                             % (i, total_count, filename,))
            unpack_path = join(CFG_TAR_FILES, filename)
            self.retrieved_packages_unpacked.append(unpack_path)
            try:
                self.ftp.download(filename, CFG_TAR_FILES)
                self.retrieved_packages_unpacked.append(unpack_path)
                self.packages_delivery.append((filename[0:-4], datetime.now()))
            except:
                register_exception(alert_admin=True,
                                   prefix="Elsevier package download failed.")
                self.logger.error("Error downloading tar file %s of %s: %s"
                                  % (i, total_count, filename,))
                print(sys.exc_info())

        return self.retrieved_packages_unpacked

    def _check_md5(self):
        import hashlib

        for filename, md5 in self.retrieved_packages.iteritems():
            our_md5 = hashlib.md5(open(join(CFG_TAR_FILES, filename))
                                  .read()).hexdigest()
            try:
                if our_md5 != md5:
                    raise MD5Error(filename)
            except MD5Error:
                register_exception(alert_admin=True,
                                   prefix="Elsevier MD5 error.")
                self.logger.error(("MD5 error: %s\n"
                                   "Original: %s, Ours: %s")
                                  % (filename, md5, our_md5,))

    def _extract_packages(self):
        """
        Extract a package in a new temporary directory.
        """
        self.path_unpacked = mkdtemp(prefix="scoap3_package_",
                                     dir=CFG_TMPSHAREDDIR)
        for path in self.retrieved_packages_unpacked:
            scoap3utils_extract_package(path, self.path_unpacked, self.logger)

        return self.path_unpacked

    def _get_text_from_journal_item(self, journal_item, tag_list):
        try:
            for tag_name in tag_list:
                """
                This loop is a iterative assignment. It 'moves' through the tag
                tree of the journal item.
                """
                journal_item = journal_item.getElementsByTagName(tag_name)[0]
            return xml_to_text(journal_item)
        except Exception:
            raise MissingTagException("One of the searched tags (%s) "
                                      "is not valid." % (", ".join(tag_list)))

    def _get_issues(self):
        if "path_unpacked" in self.__dict__:
            for name in self.files_list:
                dataset_link = join(self.path_unpacked, name.split('.')[0],
                                    'dataset.xml')

                try:
                    dataset_xml = parse(dataset_link)
                except Exception:
                    register_exception(alert_admin=True, prefix=("Elsevier error reading dataset.xml file."))
                    error_msg = "Error reading dataset.xml file: %s"
                    self.logger.error(error_msg % (dataset_link,))
                    continue

                journal_issues = dataset_xml.getElementsByTagName('journal-issue')
                if journal_issues:
                    for journal_issue in journal_issues:
                        try:
                            tag_list = ['ml', 'pathname']
                            filename = (self
                                        ._get_text_from_journal_item(journal_issue,
                                                                     tag_list))
                            self.logger.info("Found issue %s in %s."
                                             % (filename, name))
                            pathname = join(self.path_unpacked,
                                            name.split('.')[0],
                                            filename)
                            self.found_issues.append(pathname)
                        except Exception as err:
                            register_exception(alert_admin=True,
                                               prefix=err.message)
                            self.logger.error("%s", err.message)
                            continue
                else:
                    def visit(arg, dirname, names):
                        if "issue.xml" in names:
                            self.found_issues.append(join(dirname, "issue.xml"))
                    walk(join(self.path_unpacked, name.split('.')[0]), visit, None)
        return self.found_issues

    def _get_metadata_and_fulltex_dir(self):
        print("Retrieving journal items directories.")

        total_count = len(self.files_list)

        for i, name in enumerate(self.files_list, start=1):
            dataset_link = join(self.path_unpacked, name.split('.')[0],
                                'dataset.xml')

            try:
                dataset_xml = parse(dataset_link)
            except Exception:
                register_exception(alert_admin=True,
                                   prefix=("Elsevier error reading "
                                           "dataset.xml file."))
                self.logger.error("Error reading dataset.xml file: %s"
                                  % (dataset_link,))
                continue

            journal_items = dataset_xml.getElementsByTagName('journal-item')
            self.logger.info(("%s of %s: Getting metadata and fulltex "
                              "directories for %i journal items.")
                             % (i, total_count, len(journal_items),))
            for journal_item in journal_items:
                try:
                    tag_list = ['ml', 'pathname']
                    xml_pathname = join(self.path_unpacked,
                                        name.split('.')[0],
                                        (self._get_text_from_journal_item(
                                            journal_item,
                                            tag_list)))
                    tag_list = ['web-pdf', 'pathname']
                    pdf_pathname = join(self.path_unpacked,
                                        name.split('.')[0],
                                        (self._get_text_from_journal_item(
                                            journal_item,
                                            tag_list)))

                    self.found_articles.append(dict(xml=xml_pathname,
                                                    pdf=pdf_pathname,
                                                    package=name.split('.'[0])))
                except MissingTagException as err:
                    register_exception(alert_admin=True,
                                       prefix=err.message)
                    self.logger.error("%s", err.message)
                    continue

            self.logger.info(("%s of %s: Found articles: %i.")
                             % (i, total_count, len(self.found_articles),))

        self.sort_results()
        return self.found_articles

    def sort_results(self):
        self.found_articles = sorted(self.found_articles,
                                     key=lambda x: find_package_name(x['xml']),
                                     cmp=contrast_out_cmp)

    def run(self, run_localy=False):
        if not run_localy:
            try:
                self.connect()
                self._get_file_listing('.ready')
                self._download_file_listing()
            except LoginException as err:
                register_exception(alert_admin=True,
                                   prefix=('Failed to connect to '
                                           'the Elsevier server. %s') % (err,))
                return
            except Exception as e:
                self.logger.info('No new packages to process')
                self.logger.info('Registered error: %s' % e)
                return
            self._get_packages()
            self._download_tars()
            self._check_md5()
        else:
            self.logger.info("Running on local files.")
            self.retrieved_packages_unpacked = []
            self.files_list = []
            for p in listdir(CFG_TAR_FILES):
                self.retrieved_packages_unpacked.append(join(CFG_TAR_FILES, p))
            for p in listdir(CFG_READY_PACKAGES):
                self.files_list.append(p.strip(".ready.xml"))
        self._extract_packages()
        self._get_metadata_and_fulltex_dir()
